@@ -6,6 +6,7 @@ from psycopg2.extras import RealDictCursor
 from requests import HTTPError
 
 from fhir_cli import (
+    DBT_META_TABLE,
     DBT_SCHEMA,
     FHIR_COLUMN_NAME,
     POSTGRES_DB,
@@ -58,7 +59,7 @@ class Cli:
         self.admin = Admin()
 
     @staticmethod
-    def link(source: str, target: str = None):
+    def link(source: str, include: Tuple[str] = None, exclude: Tuple[str] = None):
         """Create a foreign schema from a schema of the source database
 
         This command uses a foreign data wrapper to access data
@@ -69,12 +70,17 @@ class Cli:
 
         Args:
             source (str): the source schema
-            target (:obj:`str`, optional): the target schema. If not specified,
-            the target schema will take the name of the source schema.
+            include (:obj:`tuple` of :obj:`str`, optional): comma separated table names to import
+                only foreign tables matching one of the given table names. Other tables existing in
+                the foreign schema will be ignored.
+            exclude (:obj:`tuple` of :obj:`str`, optional): comma separated table names to exclude
+                specified  foreign tables from the import. All tables existing in the foreign
+                schema will be imported except the ones listed here.
         """
 
-        if not target:
-            target = source
+        if include and exclude:
+            log.error("include and exclude options are mutually exclusive")
+            return
 
         conn = connect(
             host=POSTGRES_HOST,
@@ -84,15 +90,40 @@ class Cli:
             password=POSTGRES_PASSWORD,
         )
 
-        schema_stmt = sql.SQL("CREATE SCHEMA {target_schema}").format(
-            target_schema=sql.Identifier(target)
+        with conn.cursor() as curs:
+            db_type_stmt = sql.SQL("SELECT db_type FROM {dbt_meta_table}").format(
+                dbt_meta_table=sql.Identifier(DBT_SCHEMA, DBT_META_TABLE)
+            )
+            curs.execute(db_type_stmt)
+            (db_type) = curs.fetchone()
+
+        schema_stmt = sql.SQL("CREATE SCHEMA IF NOT EXISTS {schema}").format(
+            schema=sql.Identifier(source)
         )
+
+        constraints = sql.SQL("")
+        if include:
+            included_tables = list(map(lambda table: sql.Identifier(table), include))
+            constraints = sql.SQL("LIMIT TO ({included_tables})").format(
+                included_tables=sql.SQL(", ").join(included_tables)
+            )
+        elif exclude:
+            excluded_tables = list(map(lambda table: sql.Identifier(table), exclude))
+            constraints = sql.SQL("EXCEPT ({excluded_tables})").format(
+                excluded_tables=sql.SQL(", ").join(excluded_tables)
+            )
+
+        # increase the number of rows prefetched from the oracle db to improve perf.
+        # see https://github.com/laurenz/oracle_fdw#foreign-table-options
+        options = sql.SQL("OPTIONS (prefetch '10240')") if db_type == "oracle" else sql.SQL("")
+
         import_stmt = sql.SQL(
-            "IMPORT FOREIGN SCHEMA {source_schema} FROM SERVER source_server INTO {target_schema}"
-        ).format(
-            source_schema=sql.Identifier(source),
-            target_schema=sql.Identifier(target),
-        )
+            "IMPORT FOREIGN SCHEMA {schema} "
+            "{constraints} "
+            "FROM SERVER source_server INTO {schema} "
+            "{options}"
+        ).format(constraints=constraints, options=options, schema=sql.Identifier(source))
+
         with conn.cursor() as curs:
             curs.execute(schema_stmt)
             curs.execute(import_stmt)
